@@ -192,7 +192,7 @@ a single word in a MATCH search."
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;;;           Variables          ;;;;;
 
-(defconst dictem-version "1.0.2"
+(defconst dictem-version "1.0.4"
   "DictEm version information.")
 
 (defvar dictem-strategy-alist
@@ -437,7 +437,7 @@ This variable is local to buffer")
   (let ((dictem-temp nil))
     (loop
      (let ((line (dictem-get-line)))
-       (if (string-match "^[^ ]+" line)
+       (if (string-match "^[^ ]+:" line)
 	   (progn
 	     (if (consp dictem-temp)
 		 (setcar (cdar dictem-temp)
@@ -446,7 +446,7 @@ This variable is local to buffer")
 	      dictem-temp
 	      (cons
 	       (list
-		(substring line (match-beginning 0) (match-end 0))
+		(substring line (match-beginning 0) (- (match-end 0) 1))
 		(nreverse 
 		 (dictem-tokenize (substring line (match-end 0)))))
 	       dictem-temp)))
@@ -476,82 +476,62 @@ This variable is local to buffer")
 (defun dictem-local-dict-basic-option (host port option-mime)
   (let ((server-host (if host host (dictem-get-server))))
     (append
-     (list "-P" "-" 
-	   "--client" (dictem-client-text))
+     (list "-P" "-")
      (if server-host
 	 (list "-h" server-host "-p" (dictem-get-port port)))
      (if option-mime '("-M"))
      dictem-client-prog-args-list
      )))
 
+(defun dictem-call-process (buffer host port args)
+  (let (coding-system
+	coding-system-for-read
+	coding-system-for-write)
+    (if (and (functionp 'coding-system-list)
+	     (member 'utf-8 (coding-system-list)))
+	(setq coding-system 'utf-8))
+    (setq coding-system-for-read coding-system)
+    (setq coding-system-for-write coding-system)
+    (apply 'call-process
+	   `(,dictem-client-prog
+	     nil
+	     ,(dictem-get-buffer buffer)
+	     nil
+	     ,@(dictem-local-dict-basic-option host port nil)
+	     ,@args
+	     ))))
+
 (defun dictem-call-process-SHOW-SERVER (buffer host port)
-  (apply 'call-process
-	 `(,dictem-client-prog
-	   nil
-	   ,(dictem-get-buffer buffer)
-	   nil
-	   ,@(dictem-local-dict-basic-option host port nil)
-	   "-I")))
+  (dictem-call-process buffer host port '("-I")))
 
 (defun dictem-call-process-SHOW-INFO (buffer db host port)
-  (apply 'call-process
-	 `(,dictem-client-prog
-	   nil
-	   ,(dictem-get-buffer buffer)
-	   nil
-	   ,@(dictem-local-dict-basic-option host port nil)
-	   "-i" ,db)))
+  (dictem-call-process buffer host port (list "-i" db)))
 
 (defun dictem-call-process-SHOW-STRAT (buffer host port)
-  (apply 'call-process
-	 `(,dictem-client-prog
-	   nil
-	   ,(dictem-get-buffer buffer)
-	   nil
-	   ,@(dictem-local-dict-basic-option host port nil)
-	   "-S")))
+  (dictem-call-process buffer host port '("-S")))
 
 (defun dictem-call-process-SHOW-DB (buffer host port)
-  (apply 'call-process
-	 `(,dictem-client-prog
-	   nil
-	   ,(dictem-get-buffer buffer)
-	   nil
-	   ,@(dictem-local-dict-basic-option host port nil)
-	   "-D")))
+  (dictem-call-process buffer host port '("-D")))
 
 (defun dictem-call-process-MATCH (buffer db query strat host port)
-  (apply 'call-process
-	 `(,dictem-client-prog
-	   nil
-	   ,(dictem-get-buffer buffer)
-	   nil
-	   ,@(dictem-local-dict-basic-option host port nil)
-	   "-m"
-	   "-d" ,(if db db "*")
-	   "-s" ,(if strat strat ".")
-	   ,query)))
+  (dictem-call-process
+   buffer host port
+   (list "-m"
+	 "-d" (if db db "*")
+	 "-s" (if strat strat ".")
+	 query)))
 
 (defun dictem-call-process-DEFINE (buffer db query host port)
-  (apply 'call-process
-	 `(,dictem-client-prog
-	   nil
-	   ,(dictem-get-buffer buffer)
-	   nil
-	   ,@(dictem-local-dict-basic-option host port dictem-option-mime)
-	   "-d" ,(if db db "*")
-	   ,query)))
+  (dictem-call-process
+   buffer host port
+   (list "-d" (if db db "*") query)))
 
 (defun dictem-call-process-SEARCH (buffer db query strat host port)
-  (apply 'call-process
-	 `(,dictem-client-prog
-	   nil
-	   ,(dictem-get-buffer buffer)
-	   nil
-	   ,@(dictem-local-dict-basic-option host port dictem-option-mime)
-	   "-d" ,(if db db "*")
-	   "-s" ,(if strat strat ".")
-	   ,query)))
+  (dictem-call-process
+   buffer host port
+   (list "-d" (if db db "*")
+	 "-s" (if strat strat ".")
+	 query)))
 
 ;;;;;        GET Functions         ;;;;;
 
@@ -747,7 +727,7 @@ See dictem-exclude-databases variable"
 (defun dictem-tokenize (s)
   (if (string-match "\"[^\"]+\"\\|[^ \"]+" s )
 ;	(substring s (match-beginning 0) (match-end 0))
-      (cons (substring s (match-beginning 0) (match-end 0)) 
+      (cons (substring s (match-beginning 0) (match-end 0))
 	    (dictem-tokenize (substring s (match-end 0))))
     nil))
 
@@ -833,7 +813,6 @@ and run (error ...) if an initialization fails"
 (defun dictem-select-strategy (&optional default-strat)
   "Switches to minibuffer and asks the user
 to enter a search strategy."
-  (interactive)
   (dictem-reinitialize-err)
   (dictem-select
    "strategy"
@@ -845,7 +824,6 @@ to enter a search strategy."
 (defun dictem-select-database (spec-dbs user-dbs &optional default-db)
   "Switches to minibuffer and asks user
 to enter a database name."
-  (interactive)
   (dictem-reinitialize-err)
   (let* ((dbs (dictem-remove-value-from-alist dictem-database-alist))
 	 (dbs2 (if user-dbs
@@ -862,7 +840,6 @@ to enter a database name."
 
 (defun dictem-read-query (&optional default-query)
   "Switches to minibuffer and asks user to enter a query."
-  (interactive)
   (if (featurep 'xemacs)
       (read-string
        (concat "query [" default-query "]: ")
@@ -951,6 +928,7 @@ to enter a database name."
 	 (databases    nil)
 	 (user-db      (assoc database dictem-user-databases-alist))
 	 )
+    (goto-char (point-max))
     (cond ((dictem-userdb-p database)
 	   (apply 'dictem-base-do-default-server
 		  (append (list cmd hook database) args)))
@@ -960,7 +938,8 @@ to enter a database name."
 	    `(lambda (db)
 	       (apply 'dictem-base-do-selector 
 		      (append (list ,cmd hook db) args)))
-	    (cdr database)))
+	    (cdr database))
+	   (setq dictem-last-database (car database)))
 
 	  ((and database (stringp database)
 		(setq splitted-url (dictem-parse-url database)))
@@ -1139,7 +1118,6 @@ to enter a database name."
 
 (defun dictem-run (search-fun &optional database query strategy)
   "Creates new *dictem* buffer and run search-fun"
-  (interactive)
 
   (let ((ex_status -1))
 
@@ -1158,59 +1136,58 @@ to enter a database name."
        )
       ex_status)
 
-    (let ((coding-system nil))
-      (if (and (functionp 'coding-system-list)
-	       (member 'utf-8 (coding-system-list)))
-	  (setq coding-system 'utf-8))
-      (let ((selected-window (frame-selected-window))
-	    (coding-system-for-read coding-system)
-	    (coding-system-for-write coding-system)
+    (let ((selected-window (frame-selected-window))
 	    ; here we remember values of variables local to buffer
-	    (server           dictem-server)
-	    (port             dictem-port)
-	    (dbs              dictem-database-alist)
-	    (strats           dictem-strategy-alist)
-	    (user-dbs         dictem-user-databases-alist)
-	    (user-only        dictem-use-user-databases-only)
-	    (use-existing-buf dictem-use-existing-buffer)
+	  (server           dictem-server)
+	  (port             dictem-port)
+	  (dbs              dictem-database-alist)
+	  (strats           dictem-strategy-alist)
+	  (user-dbs         dictem-user-databases-alist)
+	  (user-only        dictem-use-user-databases-only)
+	  (use-existing-buf dictem-use-existing-buffer)
 ;	    (option-mime      dictem-option-mime)
-	    (dict-buf         nil)
-	    )
-	(if dictem-use-existing-buffer
-	    (dictem-ensure-buffer)
-	  (dictem))
-	(setq dict-buf (buffer-name))
+	  (dict-buf         nil)
+	  )
+      (cond
+       ((eq dictem-use-existing-buffer 'always)
+	(dictem-ensure-buffer))
+       ((eq dictem-use-existing-buffer t)
+	(dictem-ensure-buffer))
+       (t
+	(dictem))
+       0)
+      (setq dict-buf (buffer-name))
 ;	(set-buffer-file-coding-system coding-system)
-	(make-local-variable 'dictem-default-strategy)
-	(make-local-variable 'dictem-default-database)
-	(make-local-variable 'case-replace)
-	(make-local-variable 'case-fold-search)
+      (make-local-variable 'dictem-default-strategy)
+      (make-local-variable 'dictem-default-database)
+      (make-local-variable 'case-replace)
+      (make-local-variable 'case-fold-search)
 
 	; the following lines are to inherit values local to buffer
-	(set (make-local-variable 'dictem-server) server)
-	(set (make-local-variable 'dictem-port)   port)
-	(set (make-local-variable 'dictem-database-alist) dbs)
-	(set (make-local-variable 'dictem-strategy-alist) strats)
-	(set (make-local-variable 'dictem-user-databases-alist) user-dbs)
-	(set (make-local-variable 'dictem-use-user-databases-only) user-only)
-	(set (make-local-variable 'dictem-use-existing-buffer) use-existing-buf)
+      (set (make-local-variable 'dictem-server) server)
+      (set (make-local-variable 'dictem-port)   port)
+      (set (make-local-variable 'dictem-database-alist) dbs)
+      (set (make-local-variable 'dictem-strategy-alist) strats)
+      (set (make-local-variable 'dictem-user-databases-alist) user-dbs)
+      (set (make-local-variable 'dictem-use-user-databases-only) user-only)
+      (set (make-local-variable 'dictem-use-existing-buffer) use-existing-buf)
 
 ;	(set (make-local-variable 'dictem-option-mime) option-mime)
 
-	(set (make-local-variable 'dictem-hyperlinks-alist) nil)
+      (set (make-local-variable 'dictem-hyperlinks-alist) nil)
 
 	;;;;;;;;;;;;;;
-	(setq case-replace nil)
-	(setq case-fold-search nil)
-	(setq dictem-error-messages nil)
-	(dictem-local-run-functions search-fun database query strategy)
-	(switch-to-buffer dict-buf)
-	(if (and (not (equal ex_status 0)) (= (point-min) (point-max)))
-	    (insert (dictem-generate-full-error-message ex_status)))
-	(goto-char (point-min))
-	(setq buffer-read-only t)
-	ex_status
-	))))
+      (setq case-replace nil)
+      (setq case-fold-search nil)
+      (setq dictem-error-messages nil)
+      (dictem-local-run-functions search-fun database query strategy)
+      (switch-to-buffer dict-buf)
+      (if (and (not (equal ex_status 0)) (= (point-min) (point-max)))
+	  (insert (dictem-generate-full-error-message ex_status)))
+      (goto-char (point-min))
+      (setq buffer-read-only t)
+      ex_status
+      )))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 (defun dictem-next-section ()
@@ -1732,94 +1709,109 @@ the function 'dictem-postprocess-definition-hyperlinks'")
 		 )))))
     ))
 
-(defun dictem-postprocess-definition-hyperlinks ()
+(defun dictem-find-brackets (re-beg re-end)
+  (let ((beg-beg (make-marker))
+	(beg-end (make-marker))
+	(end-beg (make-marker))
+	(end-end (make-marker)))
+    (if (search-forward-regexp re-beg nil t)
+	(progn
+	  (set-marker beg-beg (match-beginning 0))
+	  (set-marker beg-end (match-end 0))
+	  (if (search-forward-regexp re-end nil t)
+	      (progn
+		(set-marker end-beg (match-beginning 0))
+		(set-marker end-end (match-end 0))
+		(list beg-beg beg-end end-beg end-end))
+	    nil))
+      nil)))
+
+(defun dictem-postprocess-definition-hyperlinks-cyrlybr1 ()
+  (save-excursion
+    (goto-char (point-min))
+    (let ((regexp) (pos) (beg1) (beg2) (beg3) (end) (word))
+
+      (while (setq pos (dictem-find-brackets dictem-hyperlink-beginning
+					     dictem-hyperlink-end))
+	(delete-region (nth 0 pos) (nth 1 pos))
+	(delete-region (nth 2 pos) (nth 3 pos))
+	(setq word (buffer-substring-no-properties (nth 1 pos) (nth 2 pos)))
+	(dictem-create-link
+	 (nth 1 pos) (nth 2 pos)
+	 'dictem-reference-definition-face
+	 dictem-hyperlink-define-func
+	 (list (cons 'word (dictem-replace-spaces word))
+	       (cons 'dbname dictem-current-dbname))
+	 '(link t))))))
+
+(defun dictem-postprocess-definition-hyperlinks-curlybr2 ()
   (save-excursion
     (goto-char (point-min))
     (let ((regexp
-	   (concat dictem-hyperlink-beginning "\\([^{}|]+\\)" dictem-hyperlink-end
-		   "\\|"
-		   "^From [^\n]+\\[\\([^\n]+\\)\\]"
-		   "\\|"
-		   "\\(" dictem-hyperlink-beginning "\\([^{}|\n]+\\)|\\([^{}|\n]+\\)"
-		   dictem-hyperlink-end "\\)")))
+	   (concat dictem-hyperlink-beginning "\\([^{}|\n]+\\)|\\([^{}|\n]+\\)"
+		   dictem-hyperlink-end)))
 
       (while (search-forward-regexp regexp nil t)
-	(cond ((match-beginning 1)
-	       (let* ((beg (match-beginning 1))
-		      (end (match-end 1))
-		      (match-beg (match-beginning 0))
-		      (word (buffer-substring-no-properties beg end)))
-		 (replace-match word t t)
-		 (dictem-create-link
-		  match-beg (+ match-beg (length word))
-		  'dictem-reference-definition-face
-		  dictem-hyperlink-define-func
-		  (list (cons 'word (dictem-replace-spaces word))
-			(cons 'dbname dictem-current-dbname))
-		  '(link t))
-		 ))
-	      ((match-beginning 2)
-	       (if (null dictem-current-dbname)
-		   (setq dictem-current-dbname
-			 (dictem-replace-spaces
-			  (buffer-substring-no-properties (match-beginning 2)
-							  (match-end 2))))))
-	      ((match-beginning 3)
-	       (let* ((beg (match-beginning 5))
-		      (end (match-end 5))
-		      (match-beg (match-beginning 3))
-	              (repl-beg (match-beginning 4))
-		      (repl-end (match-end 4))
-		      (repl (buffer-substring-no-properties repl-beg repl-end))
-		      (word (buffer-substring-no-properties beg end)))
-		 (replace-match repl t t)
-		 (dictem-create-link
-		  match-beg (+ match-beg (length repl))
-		  'dictem-reference-definition-face
-		  dictem-hyperlink-define-func
-		  (list (cons 'word (dictem-replace-spaces word))
-			(cons 'dbname dictem-current-dbname))
-		  '(link t))))
-	      )))))
+	(let* ((beg (match-beginning 5))
+	       (end (match-end 5))
+	       (match-beg (match-beginning 3))
+	       (repl-beg (match-beginning 4))
+	       (repl-end (match-end 4))
+	       (repl (buffer-substring-no-properties repl-beg repl-end))
+	       (word (buffer-substring-no-properties beg end)))
+	  (replace-match repl t t)
+	  (dictem-create-link
+	   match-beg (+ match-beg (length repl))
+	   'dictem-reference-definition-face
+	   dictem-hyperlink-define-func
+	   (list (cons 'word (dictem-replace-spaces word))
+		 (cons 'dbname dictem-current-dbname))
+	   '(link t)))))))
+
+(defun dictem-postprocess-definition-hyperlinks ()
+  (dictem-postprocess-definition-hyperlinks-cyrlybr1)
+  (dictem-postprocess-definition-hyperlinks-curlybr2)
+;  (dictem-postprocess-definition-hyperlinks-curlybr2)
+  )
 
 (defun dictem-postprocess-match ()
-  (goto-char (point-min))
-  (let ((last-database dictem-last-database)
-	(regexp "\\(\"[^\"\n]+\"\\)\\|\\([^ \"\n]+\\)"))
+  (save-excursion
+    (goto-char (point-min))
+    (let ((last-database dictem-last-database)
+	  (regexp "\\(\"[^\"\n]+\"\\)\\|\\([^ \"\n]+\\)"))
 
-;    (forward-line-nomark)
-    (while (search-forward-regexp regexp nil t)
-      (let* ((beg (match-beginning 0))
-	     (end (match-end 0))
-	     (first-char (buffer-substring-no-properties beg beg)))
-	(cond
-	 ((save-excursion (goto-char beg) (= 0 (current-column)))
-	  (setq last-database
-		(dictem-replace-spaces
-		 (buffer-substring-no-properties beg (- end 1))))
-	  (dictem-create-link
-	   beg (- end 1)
-	   'dictem-reference-dbname-face 'dictem-base-show-info
-	   (list (cons 'dbname last-database))))
-	 ((match-beginning 1)
-	  (dictem-create-link
-	   beg end
-	   'dictem-reference-m1-face 'dictem-base-define
-	   (list (cons 'word
-		       (dictem-replace-spaces
-			(buffer-substring-no-properties
-			 (+ beg 1) (- end 1))))
-		 (cons 'dbname last-database))))
-	 (t
-	  (dictem-create-link
-	   beg end
-	   'dictem-reference-m2-face 'dictem-base-define
-	   (list (cons 'word
-		       (dictem-replace-spaces
-			(buffer-substring-no-properties
-			 beg end )))
-		 (cons 'dbname last-database))))
-	 )))))
+      (while (search-forward-regexp regexp nil t)
+	(let* ((beg (match-beginning 0))
+	       (end (match-end 0))
+	       (first-char (buffer-substring-no-properties beg beg)))
+	  (cond
+	   ((save-excursion (goto-char beg) (= 0 (current-column)))
+	    (setq last-database
+		  (dictem-replace-spaces
+		   (buffer-substring-no-properties beg (- end 1))))
+	    (dictem-create-link
+	     beg (- end 1)
+	     'dictem-reference-dbname-face 'dictem-base-show-info
+	     (list (cons 'dbname last-database))))
+	   ((match-beginning 1)
+	    (dictem-create-link
+	     beg end
+	     'dictem-reference-m1-face 'dictem-base-define
+	     (list (cons 'word
+			 (dictem-replace-spaces
+			  (buffer-substring-no-properties
+			   (+ beg 1) (- end 1))))
+		   (cons 'dbname last-database))))
+	   (t
+	    (dictem-create-link
+	     beg end
+	     'dictem-reference-m2-face 'dictem-base-define
+	     (list (cons 'word
+			 (dictem-replace-spaces
+			  (buffer-substring-no-properties
+			   beg end )))
+		   (cons 'dbname last-database))))
+	   ))))))
 
 (defun dictem-postprocess-definition-remove-header ()
   (save-excursion
@@ -1834,6 +1826,113 @@ the function 'dictem-postprocess-definition-hyperlinks'")
 	      (kill-line 1))
 	    )))))
 
+(defun dictem-add-text-face-properties (start end face-add-props
+					      &optional object)
+  (let (face-props)
+    (while (<= start end)
+      (progn
+	(setq face-props (get-text-property start 'face object))
+	(if (facep face-props)
+	    (progn
+	      (setq face-props nil)
+	      (add-text-properties
+	       start (+ 1 start)
+	       (list 'face nil)
+	       object)))
+	(add-text-properties
+	 start (+ 1 start)
+	 (list 'face (append face-props face-add-props))
+	 object)
+	(setq start (+ start 1))))))
+
+(defun dictem-add-begendre-face-propertires (re-beg re-end face-properties)
+  (let ((bold-beg-beg (make-marker))
+	(bold-beg-end (make-marker))
+	(bold-end-beg (make-marker))
+	(bold-end-end (make-marker)))
+    (while (search-forward-regexp re-beg nil t)
+      (progn
+	(set-marker bold-beg-beg (match-beginning 0))
+	(set-marker bold-beg-end (match-end 0))
+	(if (search-forward-regexp re-end nil t)
+	    (progn
+	      (set-marker bold-end-beg (match-beginning 0))
+	      (set-marker bold-end-end (match-end 0))
+	      (dictem-add-text-face-properties
+	       bold-beg-end (- bold-end-beg 1) face-properties)
+	      (delete-region bold-beg-beg bold-beg-end)
+	      (delete-region bold-end-beg bold-end-end)
+	      ))))))
+
+(defun dictem-postprocess-stardict-definition ()
+  (interactive)
+
+  (goto-char (point-min))
+  (dictem-add-begendre-face-propertires
+   "<b>" "</b>" '(:weight bold))
+
+  (goto-char (point-min))
+  (dictem-add-begendre-face-propertires
+   "<k>" "</k>" '(:height 1.2 :foreground "white" :weight bold))
+
+  (goto-char (point-min))
+  (dictem-add-begendre-face-propertires
+   "<abr>" "</abr>" '(:weight bold :foreground "green"))
+
+  (goto-char (point-min))
+  (dictem-add-begendre-face-propertires
+   "<dtrn>" "</dtrn>" '())
+
+  (goto-char (point-min))
+  (dictem-add-begendre-face-propertires
+   "<c c=\"green\">" "</c>" '(:foreground "green"))
+
+  (goto-char (point-min))
+  (dictem-add-begendre-face-propertires
+   "<c c=\"brown\">" "</c>" '(:foreground "brown"))
+
+  (goto-char (point-min))
+  (dictem-add-begendre-face-propertires
+   "<c>" "</c>" '(:foreground "green"))
+
+  (goto-char (point-min))
+  (dictem-add-begendre-face-propertires
+   "<ex>" "</ex>" '(:foreground "BurlyWood"))
+
+  (goto-char (point-min))
+  (dictem-add-begendre-face-propertires
+   "<i>" "</i>" '(:slant "oblique"))
+
+  (goto-char (point-min))
+  (dictem-add-begendre-face-propertires
+   "<c c=\"blueviolet\">" "</c>" '(:foreground "lightblue"))
+
+  ; replaceing <tr> with [
+  (goto-char (point-min))
+  (while (search-forward-regexp "<tr>" nil t)
+    (replace-match "[" t t))
+
+  ; replaceing </tr> with ]
+  (goto-char (point-min))
+  (while (search-forward-regexp "</tr>" nil t)
+    (replace-match "]" t t))
+
+  ; replaceing <co> with (
+  (goto-char (point-min))
+  (while (search-forward-regexp "<co>" nil t)
+    (replace-match "" t t))
+
+  ; replaceing </co> with (
+  (goto-char (point-min))
+  (while (search-forward-regexp "</co>" nil t)
+    (replace-match "" t t))
+
+  (let ((dictem-hyperlink-beginning "<kref>")
+	(dictem-hyperlink-end "</kref>"))
+    (dictem-postprocess-definition-hyperlinks-cyrlybr1))
+
+  )
+
 ;;;;;       On-Click Functions     ;;;;;
 (defun dictem-define-on-press ()
   "Is called upon pressing Enter."
